from urlparse import urlparse

from django.conf import settings

from cumulus.storage import CloudFilesStorage, CloudFilesStaticStorage

def _is_ssl_uri(uri):
    return urlparse(uri).scheme == 'https'

def _get_container_urls(cloudfiles_storage):
    cdn_url = cloudfiles_storage._get_container_url()
    ssl_url = cdn_url if _is_ssl_uri(cdn_url) else cloudfiles_storage.container.public_ssl_uri()
    
    return cdn_url, ssl_url

def cdn_url(request):
    """
    A context processor to expose the full CDN URL in templates.
    """
    cdn_url, ssl_url = _get_container_urls(CloudFilesStorage())
    static_url = settings.STATIC_URL

    return {'CDN_URL': cdn_url+static_url, 'CDN_SSL_URL': ssl_url+static_url}

def static_cdn_url(request):
    """
    A context processor to expose the full static CDN URL 
    as static URL in templates.
    """
    cdn_url, ssl_url = _get_container_urls(CloudFilesStaticStorage())
    static_url = settings.STATIC_URL

    return {'STATIC_URL': cdn_url+static_url, 'STATIC_SSL_URL': ssl_url+static_url, 'LOCAL_STATIC_URL': static_url}



import mimetypes
import os
import re
import hashlib
from gzip import GzipFile
from StringIO import StringIO

import cloudfiles
from cloudfiles.errors import NoSuchObject, ResponseError

from django.core.files.base import File, ContentFile
from django.core.files.storage import Storage

from .settings import CUMULUS


HEADER_PATTERNS = tuple((re.compile(p), h) for p, h in CUMULUS.get('HEADERS', {}))


def sync_headers(cloud_obj, headers={}, header_patterns=HEADER_PATTERNS):
    """
    Overwrite the given cloud_obj's headers with the ones given as ``headers`
    and add additional headers as defined in the HEADERS setting depending on 
    the cloud_obj's file name.
    """
    # don't set headers on directories
    content_type = getattr(cloud_obj, 'content_type', None)
    if content_type == 'application/directory':
        return
    matched_headers = {}
    for pattern, pattern_headers in header_patterns:
        if pattern.match(cloud_obj.name):
            matched_headers.update(pattern_headers.copy())
    matched_headers.update(cloud_obj.headers)  # preserve headers already set
    matched_headers.update(headers)  # explicitly set headers overwrite matches and already set headers
    if matched_headers != cloud_obj.headers:
        cloud_obj.headers = matched_headers
        cloud_obj.sync_metadata()


def get_gzipped_contents(input_file):
    """
    Return a gzipped version of a previously opened file's buffer.
    """
    zbuf = StringIO()
    zfile = GzipFile(mode='wb', compresslevel=6, fileobj=zbuf)
    zfile.write(input_file.read())
    zfile.close()
    return ContentFile(zbuf.getvalue())


class CloudFilesStorage(Storage):
    """
    Custom storage for Rackspace Cloud Files.
    """
    default_quick_listdir = True
    api_key = CUMULUS['API_KEY']
    auth_url = CUMULUS['AUTH_URL']
    connection_kwargs = {}
    container_name = CUMULUS['CONTAINER']
    timeout = CUMULUS['TIMEOUT']
    use_servicenet = CUMULUS['SERVICENET']
    username = CUMULUS['USERNAME']
    ttl = CUMULUS['TTL']
    use_ssl = CUMULUS['USE_SSL']

    def __init__(self, username=None, api_key=None, container=None, timeout=None,
                 connection_kwargs=None, container_uri=None):
        """
        Initialize the settings for the connection and container.
        """
        if username is not None:
            self.username = username
        if api_key is not None:
            self.api_key = api_key
        if container is not None:
            self.container_name = container
        if timeout is not None:
            self.timeout = timeout
        if connection_kwargs is not None:
            self.connection_kwargs = connection_kwargs

        if container_uri is not None:
            self._container_public_uri = container_uri
        elif 'CONTAINER_URI' in CUMULUS:
            self._container_public_uri = CUMULUS['CONTAINER_URI']

    def __getstate__(self):
        """
        Return a picklable representation of the storage.
        """
        return dict(username=self.username,
                    api_key=self.api_key,
                    container_name=self.container_name,
                    timeout=self.timeout,
                    use_servicenet=self.use_servicenet,
                    connection_kwargs=self.connection_kwargs)

    def _get_connection(self):
        if not hasattr(self, '_connection'):
            self._connection = cloudfiles.get_connection(
                                  username=self.username,
                                  api_key=self.api_key,
                                  authurl = self.auth_url,
                                  timeout=self.timeout,
                                  servicenet=self.use_servicenet,
                                  **self.connection_kwargs)
        return self._connection

    def _set_connection(self, value):
        self._connection = value

    connection = property(_get_connection, _set_connection)

    def _get_container(self):
        if not hasattr(self, '_container'):
            self.container = self.connection.get_container(
                                                        self.container_name)
        return self._container

    def _set_container(self, container):
        """
        Set the container (and, if needed, the configured TTL on it), making
        the container publicly available.
        """
        if container.cdn_ttl != self.ttl or not container.is_public():
            container.make_public(ttl=self.ttl)
        if hasattr(self, '_container_public_uri'):
            delattr(self, '_container_public_uri')
        self._container = container

    container = property(_get_container, _set_container)

    def _get_container_url(self):
        if not hasattr(self, '_container_public_uri'):
            if self.use_ssl:
                self._container_public_uri = self.container.public_ssl_uri()
            else:
                self._container_public_uri = self.container.public_uri()
        if CUMULUS['CNAMES'] and self._container_public_uri in CUMULUS['CNAMES']:
            self._container_public_uri = CUMULUS['CNAMES'][self._container_public_uri]
        return self._container_public_uri

    container_url = property(_get_container_url)

    def _get_cloud_obj(self, name):
        """
        Helper function to get retrieve the requested Cloud Files Object.
        """
        return self.container.get_object(name)

    def _open(self, name, mode='rb'):
        """
        Return the CloudFilesStorageFile.
        """
        return CloudFilesStorageFile(storage=self, name=name)

    def _save(self, name, content):
        """
        Use the Cloud Files service to write ``content`` to a remote file
        (called ``name``).
        """
        (path, last) = os.path.split(name)
 
        # Avoid infinite loop if path is '/'
        if path and path != '/':
            try:
                self.container.get_object(path)
            except NoSuchObject:
                self._save(path, CloudStorageDirectory(path))

        content.open()
        cloud_obj = self.container.create_object(name)
        
        # If the objects has a hash, it already exists. The hash is md5 of
        # the content. If the hash has not changed, do not send the file over
        # again.
        upload = True
        if cloud_obj.etag:
            if cloud_obj.etag == cloud_obj.compute_md5sum(content.file):
                upload = False

        if upload:
            # If the content type is available, pass it in directly rather than
            # getting the cloud object to try to guess.
            if hasattr(content.file, 'content_type'):
                cloud_obj.content_type = content.file.content_type
            elif hasattr(content, 'content_type'):
                cloud_obj.content_type = content.content_type
            else:
                mime_type, encoding = mimetypes.guess_type(name)
                cloud_obj.content_type = mime_type
            # gzip the file if its of the right content type
            if cloud_obj.content_type in CUMULUS.get('GZIP_CONTENT_TYPES', []):
                content = get_gzipped_contents(content)
                cloud_obj.headers['Content-Encoding'] = 'gzip'
            # set file size
            if hasattr(content.file, 'size'):
                cloud_obj.size = content.file.size
            else:
                cloud_obj.size = content.size
            cloud_obj.send(content)

        content.close()
        sync_headers(cloud_obj)
        return name

    def delete(self, name):
        """
        Deletes the specified file from the storage system.
        """
        try:
            self.container.delete_object(name)
        except ResponseError, exc:
            if exc.status == 404:
                pass
            else:
                raise

    def exists(self, name):
        """
        Returns True if a file referenced by the given name already exists in
        the storage system, or False if the name is available for a new file.
        """
        try:
            self._get_cloud_obj(name)
            return True
        except NoSuchObject:
            return False

    def listdir(self, path):
        """
        Lists the contents of the specified path, returning a 2-tuple; the
        first being an empty list of directories (not available for quick-
        listing), the second being a list of filenames.

        If the list of directories is required, use the full_listdir method.
        """
        files = []
        if path and not path.endswith('/'):
            path = '%s/' % path
        path_len = len(path)
        for name in self.container.list_objects(path=path):
            files.append(name[path_len:])
        return ([], files)

    def full_listdir(self, path):
        """
        Lists the contents of the specified path, returning a 2-tuple of lists;
        the first item being directories, the second item being files.

        On large containers, this may be a slow operation for root containers
        because every single object must be returned (cloudfiles does not
        provide an explicit way of listing directories).
        """
        dirs = set()
        files = []
        if path and not path.endswith('/'):
            path = '%s/' % path
        path_len = len(path)
        for name in self.container.list_objects(prefix=path):
            name = name[path_len:]
            slash = name[1:-1].find('/') + 1
            if slash:
                dirs.add(name[:slash])
            elif name:
                files.append(name)
        dirs = list(dirs)
        dirs.sort()
        return (dirs, files)

    def size(self, name):
        """
        Returns the total size, in bytes, of the file specified by name.
        """
        return self._get_cloud_obj(name).size

    def url(self, name):
        """
        Returns an absolute URL where the file's contents can be accessed
        directly by a web browser.
        """
        return '%s/%s' % (self.container_url, name)

    def modified_time(self, name):
        # CloudFiles return modified date in different formats
        # depending on whether or not we pre-loaded objects.
        # When pre-loaded, timezone is not included but we
        # assume UTC. Since FileStorage returns localtime, and
        # collectstatic compares these dates, we need to depend 
        # on dateutil to help us convert timezones.
        try:
           from dateutil import parser, tz
        except ImportError:
            raise NotImplementedError("This functionality requires dateutil to be installed")

        obj = self.container.get_object(name)

        # convert to string to date
        date = parser.parse(obj.last_modified)

        # if the date has no timzone, assume UTC
        if date.tzinfo == None:
            date = date.replace(tzinfo=tz.tzutc())

        # convert date to local time w/o timezone
        date = date.astimezone(tz.tzlocal()).replace(tzinfo=None)
        return date


class CloudStorageDirectory(File):
    """
    A File-like object that creates a directory at cloudfiles
    """

    def __init__(self, name):
        super(CloudStorageDirectory, self).__init__(StringIO(), name=name)
        self.file.content_type = 'application/directory'
        self.size = 0

    def __str__(self):
        return 'directory'

    def __nonzero__(self):
        return True

    def open(self, mode=None):
        self.seek(0)

    def close(self):
        pass


class CloudFilesStaticStorage(CloudFilesStorage):
    """
    Subclasses CloudFilesStorage to automatically set the container to the one
    specified in CUMULUS['STATIC_CONTAINER']. This provides the ability to
    specify a separate storage backend for Django's collectstatic command.

    To use, make sure CUMULUS['STATIC_CONTAINER'] is set to something other
    than CUMULUS['CONTAINER']. Then, tell Django's staticfiles app by setting
    STATICFILES_STORAGE = 'cumulus.storage.CloudFilesStaticStorage'.
    """
    container_name = CUMULUS['STATIC_CONTAINER']


class CloudFilesStorageFile(File):
    closed = False

    def __init__(self, storage, name, *args, **kwargs):
        self._storage = storage
        self._pos = 0
        super(CloudFilesStorageFile, self).__init__(file=None, name=name,
                                                    *args, **kwargs)

    def _get_pos(self):
        return self._pos

    def _get_size(self):
        if not hasattr(self, '_size'):
            self._size = self._storage.size(self.name)
        return self._size

    def _set_size(self, size):
        self._size = size

    size = property(_get_size, _set_size)

    def _get_file(self):
        if not hasattr(self, '_file'):
            self._file = self._storage._get_cloud_obj(self.name)
            self._file.tell = self._get_pos
        return self._file

    def _set_file(self, value):
        if value is None:
            if hasattr(self, '_file'):
                del self._file
        else:
            self._file = value

    file = property(_get_file, _set_file)

    def read(self, num_bytes=0):
        if self._pos == self._get_size():
            return ""
        if num_bytes and self._pos + num_bytes > self._get_size():
            num_bytes = self._get_size() - self._pos
        data = self.file.read(size=num_bytes or -1, offset=self._pos)
        self._pos += len(data)
        return data

    def open(self, *args, **kwargs):
        """
        Open the cloud file object.
        """
        self._pos = 0

    def close(self, *args, **kwargs):
        self._pos = 0

    @property
    def closed(self):
        return not hasattr(self, '_file')

    def seek(self, pos):
        self._pos = pos

class ThreadSafeCloudFilesStorage(CloudFilesStorage):
    """
    Extends CloudFilesStorage to make it mostly thread safe.

    As long as you don't pass container or cloud objects
    between threads, you'll be thread safe.

    Uses one cloudfiles connection per thread.
    """

    def __init__(self, *args, **kwargs):
        super(ThreadSafeCloudFilesStorage, self).__init__(*args, **kwargs)

        import threading
        self.local_cache = threading.local()

    def _get_connection(self):
        if not hasattr(self.local_cache, 'connection'):
            connection = cloudfiles.get_connection(self.username,
                                    self.api_key, **self.connection_kwargs)
            self.local_cache.connection = connection

        return self.local_cache.connection

    connection = property(_get_connection, CloudFilesStorage._set_connection)

    def _get_container(self):
        if not hasattr(self.local_cache, 'container'):
            container = self.connection.get_container(self.container_name)
            self.local_cache.container = container

        return self.local_cache.container

    container = property(_get_container, CloudFilesStorage._set_container)



"""Delete a Rackspace CloudFiles container.

Rackspace will only let us delete an empty container,
so first we have to list all the items in the container,
then delete them one by one. This can take a long time
for big containers.
"""

import sys
import time
from optparse import make_option
from Queue import Empty
import multiprocessing

from django.core.management.base import BaseCommand, CommandError
import cloudfiles

from cumulus import settings

USAGE = 'django-admin.py container_delete <container_name>'


class Command(BaseCommand):
    """Delete a container"""

    option_list = BaseCommand.option_list + (
            make_option(
                '--yes',
                action='store_true',
                dest='is_yes',
                default=False,
                help='Assume Yes to confirmation question'),
            make_option(
                '--workers',
                dest='workers',
                default=50,
                help='Number of workers to start. Default 50.'),
            make_option(
                '--batch-size',
                dest='batch_size',
                default=10,
                help='How many tasks to give to each worker. Default 10.')
        )

    def __init__(self, *args, **kwargs):
        super(Command, self).__init__(*args, **kwargs)
        self.marker = None
        self.is_fetch_done = False
        self.batch_size = 0

    def handle(self, *args, **options):
        """Main"""

        if len(args) != 1:
            raise CommandError('Usage: %s' % USAGE)

        container_name = args[0]

        is_yes = options.get('is_yes')
        num_workers = int(options.get('workers'))
        self.batch_size = int(options.get('batch_size'))

        print('Connecting')
        conn = cloudfiles.get_connection(
                username=settings.CUMULUS['USERNAME'],
                api_key=settings.CUMULUS['API_KEY'],
                authurl=settings.CUMULUS['AUTH_URL'])
        container = conn.get_container(container_name)

        if not is_yes:
            is_ok = raw_input('Permanently delete container %s? [y|N]' %
                                container)
            if not is_ok == 'y':
                raise CommandError('Aborted')

        queue = multiprocessing.Queue()

        print('Listing objects in container')
        while not self.is_fetch_done and queue.qsize() < (num_workers * 2):
            sys.stdout.write('\rQueued batches: %d. Need: %d ' %
                    (queue.qsize(), (num_workers * 2)))
            sys.stdout.flush()
            self.fetch_more(container, queue)

        print('Deleting objects from container')

        procs = [multiprocessing.Process(target=delete,
                                         args=(container, queue))
                for _ in range(num_workers)]

        for proc in procs:
            proc.start()

        print('')
        while not queue.empty():
            sys.stdout.write('\rItems remaining: %d                 ' %
                                (queue.qsize() * self.batch_size))
            sys.stdout.flush()
            time.sleep(1)

            if not self.is_fetch_done:
                self.fetch_more(container, queue)

        print('Container empty. Waiting for final threads to finish.')
        for proc in procs:
            proc.join()

        print('Deleting container')
        conn.delete_container(container)

    def fetch_more(self, container, queue, limit=1000):
        """Fetch some more items from the container,
        and put them on the queue.
        """

        objs = None
        is_fetched = False
        while not is_fetched and limit > 5:
            try:
                objs = container.list_objects(limit=limit, marker=self.marker)
                is_fetched = True
            except IOError:
                is_fetched = False
                limit = limit / 2
                sys.stdout.write('\rBacking off. Limit: %d    ' % limit)
                sys.stdout.flush()
                time.sleep(1)

        if not objs:
            self.is_fetch_done = True
            return

        self.marker = objs[-1]

        # Batch them up onto queue
        while objs:
            batch = objs[:self.batch_size]
            objs = objs[self.batch_size:]

            queue.put(batch)


def delete(container, queue):
    """Delete from cloudfiles. Runs in a separate process."""

    while 1:

        try:
            batch = queue.get_nowait()
        except Empty:
            return

        for name in batch:
            try:
                container.delete_object(name)
            except Exception:
                pass




from django.core.files.uploadedfile import SimpleUploadedFile
from django.test import TestCase

from cumulus.storage import CloudFilesStorage
from cumulus.tests.models import Thing


cloudfiles_storage = CloudFilesStorage()

class CumulusTests(TestCase):
    def setUp(self):
        "Set up tiny files to test with."
        self.image = SimpleUploadedFile('1x1.jpg', '\xff\xd8\xff\xe0\x00\x10JFIF\x00\x01\x01\x00\x00\x01\x00\x01\x00\x00\xff\xfe\x00>CREATOR: gd-jpeg v1.0 (using IJG JPEG v62), default quality\n\xff\xdb\x00C\x00\x08\x06\x06\x07\x06\x05\x08\x07\x07\x07\t\t\x08\n\x0c\x14\r\x0c\x0b\x0b\x0c\x19\x12\x13\x0f\x14\x1d\x1a\x1f\x1e\x1d\x1a\x1c\x1c $.\' ",#\x1c\x1c(7),01444\x1f\'9=82<.342\xff\xdb\x00C\x01\t\t\t\x0c\x0b\x0c\x18\r\r\x182!\x1c!22222222222222222222222222222222222222222222222222\xff\xc0\x00\x11\x08\x00\x01\x00\x01\x03\x01"\x00\x02\x11\x01\x03\x11\x01\xff\xc4\x00\x1f\x00\x00\x01\x05\x01\x01\x01\x01\x01\x01\x00\x00\x00\x00\x00\x00\x00\x00\x01\x02\x03\x04\x05\x06\x07\x08\t\n\x0b\xff\xc4\x00\xb5\x10\x00\x02\x01\x03\x03\x02\x04\x03\x05\x05\x04\x04\x00\x00\x01}\x01\x02\x03\x00\x04\x11\x05\x12!1A\x06\x13Qa\x07"q\x142\x81\x91\xa1\x08#B\xb1\xc1\x15R\xd1\xf0$3br\x82\t\n\x16\x17\x18\x19\x1a%&\'()*456789:CDEFGHIJSTUVWXYZcdefghijstuvwxyz\x83\x84\x85\x86\x87\x88\x89\x8a\x92\x93\x94\x95\x96\x97\x98\x99\x9a\xa2\xa3\xa4\xa5\xa6\xa7\xa8\xa9\xaa\xb2\xb3\xb4\xb5\xb6\xb7\xb8\xb9\xba\xc2\xc3\xc4\xc5\xc6\xc7\xc8\xc9\xca\xd2\xd3\xd4\xd5\xd6\xd7\xd8\xd9\xda\xe1\xe2\xe3\xe4\xe5\xe6\xe7\xe8\xe9\xea\xf1\xf2\xf3\xf4\xf5\xf6\xf7\xf8\xf9\xfa\xff\xc4\x00\x1f\x01\x00\x03\x01\x01\x01\x01\x01\x01\x01\x01\x01\x00\x00\x00\x00\x00\x00\x01\x02\x03\x04\x05\x06\x07\x08\t\n\x0b\xff\xc4\x00\xb5\x11\x00\x02\x01\x02\x04\x04\x03\x04\x07\x05\x04\x04\x00\x01\x02w\x00\x01\x02\x03\x11\x04\x05!1\x06\x12AQ\x07aq\x13"2\x81\x08\x14B\x91\xa1\xb1\xc1\t#3R\xf0\x15br\xd1\n\x16$4\xe1%\xf1\x17\x18\x19\x1a&\'()*56789:CDEFGHIJSTUVWXYZcdefghijstuvwxyz\x82\x83\x84\x85\x86\x87\x88\x89\x8a\x92\x93\x94\x95\x96\x97\x98\x99\x9a\xa2\xa3\xa4\xa5\xa6\xa7\xa8\xa9\xaa\xb2\xb3\xb4\xb5\xb6\xb7\xb8\xb9\xba\xc2\xc3\xc4\xc5\xc6\xc7\xc8\xc9\xca\xd2\xd3\xd4\xd5\xd6\xd7\xd8\xd9\xda\xe2\xe3\xe4\xe5\xe6\xe7\xe8\xe9\xea\xf2\xf3\xf4\xf5\xf6\xf7\xf8\xf9\xfa\xff\xda\x00\x0c\x03\x01\x00\x02\x11\x03\x11\x00?\x00\xf9\xfe\x8a(\xa0\x0f\xff\xd9', content_type='image/jpeg')
        self.document = SimpleUploadedFile('test.txt', "test content")
        self.custom = SimpleUploadedFile('custom.txt', "custom type", content_type="custom/type")
        self.thing = Thing.objects.create(image=self.image, document=self.document, custom=self.custom)

    def test_file_api(self):
        "Make sure we can access file attributes using the Django File API."
        self.assertEqual(self.thing.image.width, 1)
        self.assertEqual(self.thing.image.height, 1)
        self.assertEqual(self.thing.image.size, 695)
        self.assertTrue("rackcdn.com" in self.thing.image.url,
                     "URL is not a valid Cloud Files CDN URL.")

        self.assertEqual(self.thing.document.size, 12)
        self.assertTrue("rackcdn.com" in self.thing.document.url,
                     "URL is not a valid Cloud Files CDN URL.")
        delattr(self.thing.document.storage, '_container_public_uri')
        self.thing.document.storage.use_ssl = True
        self.assertTrue(self.thing.document.url.startswith("https"))

    def test_image_content_type(self):
        "Ensure content type is set properly for the uploaded image."
        cloud_image = cloudfiles_storage.container.get_object(self.thing.image.name)
        self.assertEqual(cloud_image.content_type, "image/jpeg")

    def test_text_content_type(self):
        "Ensure content type is set properly for the uploaded text."
        cloud_doc = cloudfiles_storage.container.get_object(self.thing.document.name)
        self.assertEqual(cloud_doc.content_type, "text/plain")

    def test_custom_content_type(self):
        "Ensure content type is set properly when custom content type is supplied."
        cloud_custom = cloudfiles_storage.container.get_object(self.thing.custom.name)
        self.assertEqual(cloud_custom.content_type, "custom/type")

    def tearDown(self):
        self.thing.delete()



import cloudfiles
import optparse

from cumulus import settings
from django.core.management.base import BaseCommand, CommandError


class Command(BaseCommand):
    args = "[container_name container_name ...]"
    help = "Display info for cloud files containers"

    option_list = BaseCommand.option_list + (
        optparse.make_option('-n', '--name', action='store_true', dest='name', default=False),
        optparse.make_option('-c', '--count', action='store_true', dest='count', default=False),
        optparse.make_option('-s', '--size', action='store_true', dest='size', default=False),
        optparse.make_option('-u', '--uri', action='store_true', dest='uri', default=False)
    )

    def handle(self, *args, **options):
        conn = cloudfiles.get_connection(
                username=settings.CUMULUS['USERNAME'],
                api_key=settings.CUMULUS['API_KEY'],
                authurl=settings.CUMULUS['AUTH_URL'])

        if args:
            containers = []
            for container_name in args:
                try:
                    container = conn.get_container(container_name)
                except cloudfiles.errors.NoSuchContainer:
                    raise CommandError("Container does not exist: %s" % container_name)
                containers.append(container)
        else:
            containers = conn.get_all_containers()

        opts = ['name', 'count', 'size', 'uri']

        for container in containers:
            info = {
                'name': container.name,
                'count': container.object_count,
                'size': container.size_used,
                'uri': container.public_uri() if container.is_public() else "NOT PUBLIC",
            }
            output = [str(info[o]) for o in opts if options.get(o)]
            if not output:
                output = [str(info[o]) for o in opts]
            print ', '.join(output)

        if not containers:
            print 'No containers found.'



"""Create a public (CDN) rackspace container.
"""

import sys

from django.core.management.base import BaseCommand, CommandError
import cloudfiles

from cumulus import settings

USAGE = 'django-admin.py container_create <container_name>'


class Command(BaseCommand):
    """Create a public container"""

    def handle(self, *args, **options):
        """Main"""

        if len(sys.argv) != 3:
            raise CommandError('Usage: %s' % USAGE)

        container_name = sys.argv[2]
        print('Creating container: %s' % container_name)

        conn = cloudfiles.get_connection(
                        username=settings.CUMULUS['USERNAME'],
                        api_key=settings.CUMULUS['API_KEY'],
                        authurl=settings.CUMULUS['AUTH_URL'])

        container = conn.create_container(container_name)
        container.make_public()

        print('Done')



import datetime
import optparse
import os

import cloudfiles

from django.conf import settings
from django.core.management.base import BaseCommand
from cumulus.settings import CUMULUS
from cumulus.storage import sync_headers

class Command(BaseCommand):
    help = "Synchronizes static media to cloud files."

    option_list = BaseCommand.option_list + (
        optparse.make_option('-w', '--wipe',
            action='store_true', dest='wipe', default=False,
            help="Wipes out entire contents of container first."),
        optparse.make_option('-t', '--test-run',
            action='store_true', dest='test_run', default=False,
            help="Performs a test run of the sync."),
        optparse.make_option('-c', '--container',
            dest='container', help="Override STATIC_CONTAINER."),
    )

    # settings from cumulus.settings
    USERNAME         = CUMULUS['USERNAME']
    API_KEY          = CUMULUS['API_KEY']
    STATIC_CONTAINER = CUMULUS['STATIC_CONTAINER']
    USE_SERVICENET   = CUMULUS['SERVICENET']
    FILTER_LIST      = CUMULUS['FILTER_LIST']
    AUTH_URL         = CUMULUS['AUTH_URL']

    # paths
    DIRECTORY        = os.path.abspath(settings.STATIC_ROOT)
    STATIC_URL       = settings.STATIC_URL

    if not DIRECTORY.endswith('/'):
        DIRECTORY = DIRECTORY + '/'

    if STATIC_URL.startswith('/'):
        STATIC_URL = STATIC_URL[1:]

    local_object_names = []
    create_count = 0
    upload_count = 0
    update_count = 0
    skip_count = 0
    delete_count = 0
    conn = None
    container = None

    def handle(self, *args, **options):
        self.wipe = options.get('wipe')
        self.test_run = options.get('test_run')
        self.verbosity = int(options.get('verbosity'))
        if hasattr(options, 'container'):
            self.STATIC_CONTAINER = options.get('container')
        self.sync_files()

    def sync_files(self):
        self.conn = cloudfiles.get_connection(username = self.USERNAME,
                                              api_key = self.API_KEY,
                                              authurl = self.AUTH_URL,
                                              servicenet=self.USE_SERVICENET)
                                              
        try:
            self.container = self.conn.get_container(self.STATIC_CONTAINER)
        except cloudfiles.errors.NoSuchContainer:
            self.container = self.conn.create_container(self.STATIC_CONTAINER)

        if not self.container.is_public():
            self.container.make_public()

        # if -w option is provided, wipe out the contents of the container
        if self.wipe:
            if self.test_run:
                print "Wipe would delete %d objects." % self.container.object_count
            else:
                print "Deleting %d objects..." % self.container.object_count
                for cloud_obj in self.container.get_objects():
                    self.container.delete_object(cloud_obj.name)

        # walk through the directory, creating or updating files on the cloud
        os.path.walk(self.DIRECTORY, self.upload_files, "foo")

        # remove any files on remote that don't exist locally
        self.delete_files()

        # print out the final tally to the cmd line
        self.update_count = self.upload_count - self.create_count
        print
        if self.test_run:
            print "Test run complete with the following results:"
        print "Skipped %d. Created %d. Updated %d. Deleted %d." % (
            self.skip_count, self.create_count, self.update_count, self.delete_count)

    def upload_files(self, arg, dirname, names):
        # upload or skip items
        for item in names:
            if item in self.FILTER_LIST:
                continue # Skip files we don't want to sync

            file_path = os.path.join(dirname, item)
            if os.path.isdir(file_path):
                continue # Don't try to upload directories

            object_name = self.STATIC_URL + file_path.split(self.DIRECTORY)[1]
            self.local_object_names.append(object_name)

            try:
                cloud_obj = self.container.get_object(object_name)
            except cloudfiles.errors.NoSuchObject:
                cloud_obj = self.container.create_object(object_name)
                self.create_count += 1

            cloud_datetime = (cloud_obj.last_modified and
                              datetime.datetime.strptime(
                                cloud_obj.last_modified,
                                "%a, %d %b %Y %H:%M:%S %Z"
                              ) or None)
            local_datetime = datetime.datetime.utcfromtimestamp(
                                               os.stat(file_path).st_mtime)
            if cloud_datetime and local_datetime < cloud_datetime:
                self.skip_count += 1
                if self.verbosity > 1:
                    print "Skipped %s: not modified." % object_name
                continue

            if not self.test_run:
                cloud_obj.load_from_filename(file_path)
                sync_headers(cloud_obj)
            self.upload_count += 1
            if self.verbosity > 1:
                print "Uploaded", cloud_obj.name

    def delete_files(self):
        # remove any objects on the cloud that don't exist locally
        for cloud_name in self.container.list_objects():
            if cloud_name not in self.local_object_names:
                self.delete_count += 1
                if self.verbosity > 1:
                    print "Deleted %s" % cloud_name
                if not self.test_run:
                    self.container.delete_object(cloud_name)



import hashlib

from django.contrib.staticfiles.management.commands import collectstatic

from cumulus.storage import CloudFilesStorage

class Command(collectstatic.Command):

    def delete_file(self, path, prefixed_path, source_storage):
        """
        Checks if the target file should be deleted if it already exists
        """
        if isinstance(self.storage, CloudFilesStorage):
            if self.storage.exists(prefixed_path):
                try:
                    etag = self.storage._get_cloud_obj(prefixed_path).etag
                    digest = "{0}".format(hashlib.md5(source_storage.open(path).read()).hexdigest())
                    print etag, digest
                    if etag == digest:
                        self.log(u"Skipping '{0}' (not modified based on file hash)".format(path))
                        return False
                except:
                    raise
        return super(Command, self).delete_file(path, prefixed_path, source_storage)



"""List all the items in a rackspace cloudfiles container to stdout.

Usage: django-admin.py container_list <container_name>
We recommend you run it like this:

 django-admin.py container_list <container> | pv --line-mode > <container>.list

pv is Pipe Viewer: http://www.ivarch.com/programs/pv.shtml
"""

import pprint
import time

import cloudfiles
from django.core.management.base import BaseCommand
from cumulus import settings

MAX_LIMIT = 1000    # The most rackspace will give us in one request
MIN_LIMIT = 200     # The least we'll tolerate


class Command(BaseCommand):
    """Django management command"""

    def __init__(self, *args, **kwargs):
        super(Command, self).__init__(*args, **kwargs)
        self.connection = None

    def handle(self, *args, **options):
        """Main"""

        self.connection = cloudfiles.Connection(
                settings.CUMULUS['USERNAME'],
                settings.CUMULUS['API_KEY'],
                authurl=settings.CUMULUS['AUTH_URL'])

        if len(args) == 1:
            lister = ContainerLister(self.connection, args[0])
            lister.run()

        else:
            self.list_all_containers()

    def list_all_containers(self):
        """Print a list of all containers"""

        print('...Listing containers...')

        containers = self.connection.list_containers()

        print('Please specify one of the following containers:')
        pprint.pprint(containers)


class ContainerLister(object):
    """List objects in a container.

    Rackspace only returns up to 1000 at a time,
    so we keep requesting the next set until we get
    an empty one.
    """

    def __init__(self, connection, container_name):

        self.connection = connection
        self.container = connection.get_container(container_name)

        self.limit = MAX_LIMIT
        self.marker = None
        self.is_done = False

    def run(self):
        """Prints names of items in container to stdout"""

        while not self.is_done:
            self.list_more()

    def list_more(self):
        """List more items"""

        try:
            objs = self.container.list_objects(
                    limit=self.limit,
                    marker=self.marker)

            if objs:
                self.marker = objs[-1]

                for item in objs:
                    print(item)

            else:
                self.is_done = True

            if self.limit <= MIN_LIMIT:
                # Maybe rackspace is better now
                self.limit = MAX_LIMIT

        except IOError:
            # Rackspace is timing out, fetch less and pause.
            self.limit = self.limit / 2
            time.sleep(1)




#!/usr/bin/env python
import os

from setuptools import setup, find_packages

# Use the docstring of the __init__ file to be the description
short_description = " ".join(__import__('cumulus').__doc__.splitlines()).strip()

# Use part of the sphinx docs index for the long description
doc_dir = os.path.join(os.path.dirname(__file__), 'docs')
index_filename = os.path.join(doc_dir, 'index.rst')
long_description = open(index_filename).read().split('split here', 1)[1]

setup(
    name = "django-cumulus",
    version = __import__('cumulus').get_version().replace(' ', '-'),
    packages = find_packages(),
    install_requires = ['python-cloudfiles>=1.7.10'],

    author = "Rich Leland",
    author_email = "rich@richleland.com",
    license = 'BSD',
    description = short_description,
    long_description = long_description,
    url = "https://github.com/richleland/django-cumulus/",
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Framework :: Django',
    ]
)



from django.db import models

from cumulus.storage import CloudFilesStorage

cloudfiles_storage = CloudFilesStorage()

class Thing(models.Model):
    "A dummy model to use for tests."
    image = models.ImageField(storage=cloudfiles_storage, upload_to='cumulus-tests')
    document = models.FileField(storage=cloudfiles_storage, upload_to='cumulus-tests')
    custom = models.FileField(storage=cloudfiles_storage, upload_to='cumulus-tests')





import cloudfiles
from cloudfiles.consts import default_cdn_ttl

from django.conf import settings

CUMULUS = {
    'API_KEY': None,
    'AUTH_URL': 'us_authurl',
    'CNAMES': None,
    'CONTAINER': None,
    'SERVICENET': False,
    'TIMEOUT': 5,
    'TTL': default_cdn_ttl,  # 86400s (24h), python-cloudfiles default
    'USE_SSL': False,
    'USERNAME': None,
    'STATIC_CONTAINER': None,
    'FILTER_LIST': [],
    'HEADERS': {},
    'GZIP_CONTENT_TYPES': [],
}

if hasattr(settings, 'CUMULUS'):
    CUMULUS.update(settings.CUMULUS)

# set auth_url to the actual URL string in the cloudfiles module
CUMULUS['AUTH_URL'] = getattr(cloudfiles, CUMULUS['AUTH_URL'])

# backwards compatibility for old-style cumulus settings
if not hasattr(settings, 'CUMULUS') and hasattr(settings, 'CUMULUS_API_KEY'):
    import warnings
    warnings.warn(
        "settings.CUMULUS_* is deprecated; use settings.CUMULUS instead.",
        PendingDeprecationWarning
    )

    CUMULUS.update({
        'API_KEY': getattr(settings, 'CUMULUS_API_KEY'),
        'CNAMES': getattr(settings, 'CUMULUS_CNAMES', None),
        'CONTAINER': getattr(settings, 'CUMULUS_CONTAINER'),
        'SERVICENET': getattr(settings, 'CUMULUS_USE_SERVICENET', False),
        'TIMEOUT': getattr(settings, 'CUMULUS_TIMEOUT', 5),
        'TTL': getattr(settings, 'CUMULUS_TTL', default_cdn_ttl),
        'USERNAME': getattr(settings, 'CUMULUS_USERNAME'),
    })
